@@ -1,49 +1,95 @@
-###############################################################################
-## abn-methods.R ---
-## Author : Gilles Kratzer & Reinhard Furrer
-## Document created : 21/05/2019
-###############################################################################
-
-
-##-------------------------------------------------------------------------
-## abnDag
-##-------------------------------------------------------------------------
-
-# print
-
+#' Print objects of class \code{abnDag}
+#'
+#' @param x Object of class \code{abnDag}
+#' @param digits number of digits of the adjacency matrix.
+#' @param ... additional parameters. Not used at the moment.
+#'
+#' @return outputs adjacency matrix and statement of the class of \code{x}.
+#' @concept DAG
+#' @export
+#' @examples
+#' mydag <- createAbnDag(dag = ~a+b|a, data.df = data.frame("a"=1, "b"=1))
+#' print(mydag)
 print.abnDag <- function(x, digits = 3L, ...){
   print(x$dag, digits = digits)
   cat("Class 'abnDag'.\n")
   invisible(x)
 }
 
-# summary
-
+#' Prints summary statistics from an object of class \code{abnDag}
+#'
+#' @inherit infoDag params
+#' @param ... additional parameters. Not used at the moment.
+#' @concept DAG
+#' @export
+#' @examples
+#' mydag <- createAbnDag(dag = ~a+b|a, data.df = data.frame("a"=1, "b"=1))
+#' summary(mydag)
 summary.abnDag <- function(object, ...) {
   su <- infoDag(object$dag)
   return(su)
 }
 
-
-# plot
-
+#' Plots DAG from an object of class \code{abnDag}
+#'
+#' @param x Object of class \code{abnDag}
+#' @param new defaults to \code{TRUE} for opening a new plot.
+#' @param ... additional parameters. Not used at the moment.
+#'
+#' @return \code{Rgraphviz::plot}
+#' @concept DAG
+#' @export
+#' @importClassesFrom graph graphAM
+#' @importFrom grDevices dev.new dev.flush
+#' @examples
+#' mydag <- createAbnDag(dag = ~a+b|a, data.df = data.frame("a"=1, "b"=1))
+#' plot(mydag)
 plot.abnDag <- function(x, new=TRUE, ...){
-  if (new) dev.new()
-  on.exit(dev.flush())
+  invisible({
+    if (new) dev.new()
+    on.exit(dev.flush())
 
-  # Rgraphviz:
-  mygraph <- new("graphAM", adjMat = t(x$dag), edgemode = "directed")
-  g <- Rgraphviz::plot(x = mygraph)
+    # Rgraphviz:
+    mygraph <- new("graphAM", adjMat = t(x$dag), edgemode = "directed")
+    g <- Rgraphviz::plot(x = mygraph)
+  })
   invisible(g)
+
 }
 
 
 ##-------------------------------------------------------------------------
 ## abnCache
 ##-------------------------------------------------------------------------
-
-# print
-
+#' Print objects of class \code{abnCache}
+#'
+#' @param x Object of class \code{abnCache}
+#' @param digits number of digits of the results.
+#' @param ... additional parameters. Not used at the moment.
+#'
+#' @return summary statement of the class of \code{abnCache}.
+#' @concept DAG
+#' @export
+#' @examples
+#' ## Subset of the build-in dataset, see  ?ex0.dag.data
+#' mydat <- ex0.dag.data[,c("b1","b2","g1","g2","b3","g3")] ## take a subset of cols
+#'
+#' ## setup distribution list for each node
+#' mydists <- list(b1="binomial", b2="binomial", g1="gaussian",
+#'                 g2="gaussian", b3="binomial", g3="gaussian")
+#'
+#' # Structural constraints
+#' # ban arc from b2 to b1
+#' # always retain arc from g2 to g1
+#'
+#' ## parent limits
+#' max.par <- list("b1"=2, "b2"=2, "g1"=2, "g2"=2, "b3"=2, "g3"=2)
+#'
+#' ## now build the cache of pre-computed scores accordingly to the structural constraints
+#'
+#' res.c <- buildScoreCache(data.df=mydat, data.dists=mydists,
+#'                          dag.banned= ~b1|b2, dag.retained= ~g1|g2, max.parents=max.par)
+#' print(res.c)
 print.abnCache <- function(x, digits = 3, ...){
 
   cat("Number of nodes in the network: ",max(x$children), ".\n\n", sep='')
@@ -68,9 +114,11 @@ print.abnCache <- function(x, digits = 3, ...){
 ##-------------------------------------------------------------------------
 ## abnHeuristic
 ##-------------------------------------------------------------------------
-
-# print
-
+#' Print objects of class \code{abnHeuristic}
+#' @param x Object of class \code{abnHeuristic}
+#' @param digits number of digits of the results.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 print.abnHeuristic <- function(x, digits = 2L, ...){
   cat("Best DAG' score found with",x$algo,"algorithm with", x$num.searches,"different searches limited to" , x$max.steps,"steps:\n")
   print(max(unlist(x$scores)), digits=digits)
@@ -81,8 +129,12 @@ print.abnHeuristic <- function(x, digits = 2L, ...){
   invisible(x)
 }
 
-# plot
-
+#' Plot objects of class \code{abnHeuristic}
+#' @param x Object of class \code{abnHeuristic}
+#' @param ... additional parameters. Not used at the moment.
+#' @importFrom graphics par plot points title lines
+#' @importFrom grDevices rgb
+#' @export
 plot.abnHeuristic <- function(x, ...){
 
   df <- unlist(x$scores)
@@ -120,17 +172,23 @@ plot.abnHeuristic <- function(x, ...){
 ##-------------------------------------------------------------------------
 ## abnHillClimber
 ##-------------------------------------------------------------------------
-
-# print
-
+#' Print objects of class \code{abnHillClimber}
+#' @param x Object of class \code{abnHillClimber}
+#' @param digits number of digits of the results.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 print.abnHillClimber <- function(x, digits = 3L, ...){
   print(x$consensus, digits = digits)
   cat("Consensus DAG from 'search.hillclimber'  (class 'abnHillClimber').\n")
   invisible(x)
 }
 
-# plot
-
+#' Plot objects of class \code{abnHillClimber}
+#' @param x Object of class \code{abnHillClimber}
+#' @param new defaults to \code{TRUE} for opening a new plot.
+#' @param ... additional parameters. Not used at the moment.
+#' @importFrom grDevices dev.new dev.flush
+#' @export
 plot.abnHillClimber <- function(x, new=TRUE, ...){
 
   if (new) dev.new()
@@ -147,8 +205,11 @@ plot.abnHillClimber <- function(x, new=TRUE, ...){
 ## abnMostprobable
 ##-------------------------------------------------------------------------
 
-# print
-
+#' Print objects of class \code{abnMostprobable}
+#' @param x Object of class \code{abnMostprobable}
+#' @param digits number of digits of the results.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 print.abnMostprobable <- function(x, digits = 3L, ...){
 
   print(x$dag, digits = digits)
@@ -156,8 +217,10 @@ print.abnMostprobable <- function(x, digits = 3L, ...){
   invisible(x)
 }
 
-# summary
-
+#' Print summary of objects of class \code{abnMostprobable}
+#' @param object Object of class \code{abnMostprobable}
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 summary.abnMostprobable <- function(object, ...){
   cat("Optimal DAG from 'mostProbable':\n")
   print(object$dag)
@@ -166,10 +229,12 @@ summary.abnMostprobable <- function(object, ...){
   invisible( object)
 }
 
-
-
-# plot
-
+#' Plot objects of class \code{abnMostprobable}
+#' @param x Object of class \code{abnMostprobable}
+#' @param new defaults to \code{TRUE} for opening a new plot.
+#' @param ... additional parameters. Not used at the moment.
+#' @importFrom grDevices dev.new dev.flush
+#' @export
 plot.abnMostprobable <- function(x, new=TRUE, ...){
 
   if (new) dev.new()
@@ -184,15 +249,30 @@ plot.abnMostprobable <- function(x, new=TRUE, ...){
 ##-------------------------------------------------------------------------
 ## abnFit
 ##-------------------------------------------------------------------------
-
-# print
-
+#' Print objects of class \code{abnFit}
+#' @param x Object of class \code{abnFit}
+#' @param digits number of digits of the results.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 print.abnFit <- function(x, digits = 3L, ...){
 
   if(x$method=="mle"){
-    cat("The ABN model was fitted using an mle approach. The estimated coefficients are:\n\n")
-    print(x$coef, digits=digits)
-    cat("Number of nodes in the network: ",length(x$coef), ".\n", sep='')
+    cat("The ABN model was fitted using an mle approach.", fill = TRUE)
+    if(!is.null(x$group.var)){
+      cat("GLMM with the grouping variable ", x$group.var, ".", fill = TRUE, sep = "")
+      cat("The estimated fixed-effect parameters are:\n\n")
+      print(x$mu, digits=digits)
+      print(x$betas, digits=digits)
+      cat("The estimated random-effect parameters are:\n\n")
+      print(x$sigma, digits=digits)
+      print(x$sigma_alpha, digits=digits)
+      cat("Number of nodes in the network: ",length(x$abnDag$data.dists), " plus ", length(x$group.var), " grouping variable.", fill = TRUE, sep = "")
+    } else {
+      # no grouping
+      cat("The estimated coefficients are:\n\n")
+      print(x$coef, digits=digits)
+      cat("Number of nodes in the network: ",length(x$coef), ".\n", sep='')
+    }
   }
 
   if(x$method=="bayes"){
@@ -204,8 +284,11 @@ print.abnFit <- function(x, digits = 3L, ...){
   invisible(x)
 }
 
-# summary
-
+#' Print summary of objects of class \code{abnFit}
+#' @param object Object of class \code{abnFit}
+#' @param digits number of digits of the results.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 summary.abnFit <- function(object, digits = 3L, ...){
 
   if(object$method=="mle"){
@@ -237,8 +320,12 @@ summary.abnFit <- function(object, digits = 3L, ...){
   invisible(object)
 }
 
-# coef
-
+#' Print coefficients of objects of class \code{abnFit}
+#' @param object Object of class \code{abnFit}
+#' @param digits number of digits of the results.
+#' @param verbose print additional output.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 coef.abnFit <- function(object, digits = 3L, verbose = TRUE, ...){
   if(object$method=="mle"){
     cat("The ABN model was fitted using an mle approach. The estimated coefficients are:\n")
@@ -254,8 +341,12 @@ coef.abnFit <- function(object, digits = 3L, verbose = TRUE, ...){
 
 }
 
-
-
+#' Print AIC of objects of class \code{abnFit}
+#' @param object Object of class \code{abnFit}
+#' @param digits number of digits of the results.
+#' @param verbose print additional output.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 AIC.abnFit <- function(object, digits = 3L, verbose = TRUE, ...){
 
   if(object$method=="mle"){
@@ -271,10 +362,14 @@ AIC.abnFit <- function(object, digits = 3L, verbose = TRUE, ...){
   }
 
   invisible(object)
-
-
 }
 
+#' Print BIC of objects of class \code{abnFit}
+#' @param object Object of class \code{abnFit}
+#' @param digits number of digits of the results.
+#' @param verbose print additional output.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 BIC.abnFit <- function(object, digits = 3L, verbose = TRUE, ...){
 
   if(object$method=="mle"){
@@ -292,7 +387,12 @@ BIC.abnFit <- function(object, digits = 3L, verbose = TRUE, ...){
   invisible(object)
 }
 
-
+#' Print logLik of objects of class \code{abnFit}
+#' @param object Object of class \code{abnFit}
+#' @param digits number of digits of the results.
+#' @param verbose print additional output.
+#' @param ... additional parameters. Not used at the moment.
+#' @export
 logLik.abnFit <- function(object, digits = 3L, verbose = TRUE, ...){
 
   if(object$method=="mle"){
@@ -310,8 +410,11 @@ logLik.abnFit <- function(object, digits = 3L, verbose = TRUE, ...){
   invisible(object)
 }
 
-
-
+#' Print family of objects of class \code{abnFit}
+#' @param object Object of class \code{abnFit}
+#' @param ... additional parameters. Not used at the moment.
+#' @importFrom stats family
+#' @exportS3Method abn::family abnFit
 family.abnFit <- function(object, ...){
 
   cat("All link functions are canonical: \n
@@ -322,11 +425,22 @@ family.abnFit <- function(object, ...){
   invisible(object)
 }
 
-
+#' Print number of observations of objects of class \code{abnFit}
+#' @param object Object of class \code{abnFit}
+#' @param ... additional parameters. Not used at the moment.
+#' @importFrom stats nobs
+#' @exportS3Method abn::nobs abnFit
 nobs.abnFit <- function(object, ...){
   nrow(object$abnDag$data.df)
 }
 
+#' Plot objects of class \code{abnFit}
+#' @param x Object of class \code{abnFit}
+#' @param which defaults to "abnFit".
+#' @param ... additional parameters. Not used at the moment.
+#' @importFrom methods hasArg
+#' @importFrom stats fitted.values
+#' @export
 plot.abnFit <- function(x, which ="abnFit", ...){
 
   if (which != "abnFit") stop('Function type not implemented yet. Use which="abnFit"')
@@ -342,7 +456,3 @@ plot.abnFit <- function(x, which ="abnFit", ...){
   }
   invisible(g)
 }
-
-
-
-## EOF
